==> newsstar-1.2.2.tar.gz: newsstar-1.2.2/src/nntp.c <==
/*
    newsstar - advanced news fetcher
    Copyright (C) 2001-2002 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* $Id: nntp.c,v 1.5 2003/12/22 01:48:14 realh Exp $ */

#include "common.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#ifdef SUPPORT_SSL
#ifdef HAVE_OPENSSL_SSL_H
#include "openssl/ssl.h"
#else
#include "ssl.h"
#endif
#include "nscurses.h"
#endif

#include "nntp.h"

char *nntp_resolve_address(const char *name)
{
    struct hostent *hinfo;
    char **aliases, **addrs;
    
    hinfo = gethostbyname(name);
    if (!hinfo || hinfo->h_addrtype != AF_INET)
    {
	deblog_critical(_("Can't resolve address \"%s\"\n"),
	    name);
	return NULL;
    }
    deblog_info(_("Master host name: %s\n"), hinfo->h_name);
    deblog_info(_("Aliases:\n"));
    for (aliases = hinfo->h_aliases; *aliases; ++aliases)
    {
	deblog_info("\t%s\n", *aliases);
    }
    deblog_info(_("Addresses:\n"));
    for (addrs = hinfo->h_addr_list; *addrs; ++addrs)
    {
	deblog_info(_("\t%s\n"), inet_ntoa(*(struct in_addr *) *addrs));
    }
    return XSTRDUP(inet_ntoa(*(struct in_addr *) hinfo->h_addr_list[0]));
}

const char *nntp_check_response(rw_buf *sbuf, const char *message, const char
	*allow, deblog_level_t log_level)
{
    char digit[2] = " ";
    const char *response = rw_read_line(sbuf);
    
    if (response)
	digit[0] = response[0];
    if (!response || !strpbrk(digit, allow))
    {
	deblog_printf(log_level, "%s: %s\n", message,
		response ? response : _("no response from server"));
	return NULL;
    }
    else
    {
	deblog_debug("%s\n", response);
    }
    return response;
}

rw_buf *nntp_connect(const char *dotquad, int port, int thread_idx, BOOL ssl)
{
    int sock;
    rw_buf *buf;
#ifdef SUPPORT_SSL
    static BOOL ssl_initialised = FALSE;
    static SSL_CTX *ssl_ctx = NULL;
    SSL *ssl_dat = NULL;

    /* This seems safest place to call SSL_library_init, so that each process
     * calls it once instead of relying on correct state to be inherited from
     * parent
     */
    if (ssl)
    {
	if (!ssl_initialised)
	{
	    SSL_library_init();
	    ssl_initialised = TRUE;
	}

	if (!ssl_ctx)
	{
	    ssl_ctx = SSL_CTX_new(SSLv23_client_method());
	    if (!ssl_ctx)
	    {
		deblog_critical(_("Unable to create SSL context\n"));
		return NULL;
	    }
	}
    }
#else
    (void) ssl;
#endif

    deblog_info(_("Connecting to %s:%d\n"), dotquad, ntohs(port));
    sock = socket(AF_INET, SOCK_STREAM, 0);
    if (sock != -1)
    {
	struct sockaddr_in addr;

	addr.sin_family = AF_INET;
	addr.sin_port = port;
	inet_aton(dotquad, &addr.sin_addr);
	if (connect(sock, (struct sockaddr *) &addr, sizeof(addr))
		== -1)
	{
	    int terrno = errno;

	    close(sock);
	    errno = terrno;
	    sock = -1;
	}
    }
    else
    {
	deblog_info(_("Couldn't create socket\n"));
    }
    if (sock == -1)
    {
	deblog_critical(_("Can't connect to %s: %s\n"),
		dotquad, strerror(errno));
	return NULL;
    }
#ifdef SUPPORT_SSL
    if (ssl)
    {
	nscurses_show_status(thread_idx, _("Initialising SSL"));
	deblog_message(_("Making socket secure\n"));
	ssl_dat = SSL_new(ssl_ctx);
	if (!ssl_dat)
	{
	    deblog_critical(_("Unable to create SSL object\n"));
	    close(sock);
	    return NULL;
	}
	if (!SSL_set_fd(ssl_dat, sock))
	{
	    deblog_critical(_("Unable to attach SSL object to socket\n"));
	    SSL_free(ssl_dat);
	    close(sock);
	    return NULL;
	}
	if (SSL_connect(ssl_dat) != 1)
	{
	    deblog_critical(_("SSL handshake failed\n"));
	    SSL_free(ssl_dat);
	    close(sock);
	    return NULL;
	}
    }
    buf = rw_create_buf_ssl(sock, ssl_dat, TRUE, FALSE, thread_idx);
#else
    buf = rw_create_buf(sock, TRUE, FALSE, thread_idx);
#endif
    /*
    if (strcmp(dotquad, "127.0.0.1"))
	buf->scrutinise = TRUE;
    */
    if (!nntp_check_response(buf, _("Bad/no response when connecting"), "2",
		deblog_Level_Critical))
    {
	rw_close_and_free(buf);
	return NULL;
    }
    return buf;
}

BOOL nntp_authenticate(rw_buf *buf, const char *user, const char *passwd)
{
    deblog_info(_("Authenticating user %s\n"), user);
    rw_writef(buf, "AUTHINFO USER %s\r\n", user);
    if (!nntp_check_response(buf, _("User name rejected"), "23",
		deblog_Level_Critical))
    {
	return TRUE;
    }
    rw_writef(buf, "AUTHINFO PASS %s\r\n", passwd);
    return (nntp_check_response(buf, _("Password rejected"), "2",
		deblog_Level_Critical) == NULL);
}

BOOL nntp_set_mode(rw_buf *buf, const char *mode)
{
    deblog_info(_("Sending MODE %s\n"), mode);
    rw_writef(buf, "MODE %s\r\n", mode);
    return (nntp_check_response(buf, _("Bad response to MODE command"), "2",
		deblog_Level_Critical)
	    == NULL);
}

BOOL nntp_enter_group(rw_buf *buf, const char *groupname, long int *first, long
	int *last, BOOL check_response)
{
    const char *response;

    deblog_info(_("Entering group %s\n"), groupname);
    rw_writef(buf, "GROUP %s\r\n", groupname);
    if (check_response)
    {
	if ((response = nntp_check_response(buf, _("Can't enter group"), "2",
			deblog_Level_Warning)) == NULL)
	{
	    return TRUE;
	}
	if (first && last)
	{
	    sscanf(response, "%*d %*d %ld %ld", first, last);
	}
    }
    return FALSE;
}

==> newsstar-1.3.2.tar.gz: newsstar-1.3.2/src/nntp.c <==
/*
    newsstar - advanced news fetcher
    Copyright (C) 2001-2002 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "common.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#ifdef SUPPORT_SSL
#ifdef HAVE_OPENSSL_SSL_H
#include "openssl/ssl.h"
#else
#include "ssl.h"
#endif
#include "nscurses.h"
#endif

#include "nntp.h"

char *nntp_resolve_address(const char *name)
{
    struct hostent *hinfo;
    char **aliases, **addrs;
    
    hinfo = gethostbyname(name);
    if (!hinfo || hinfo->h_addrtype != AF_INET)
    {
	deblog_critical(_("Can't resolve address \"%s\"\n"),
	    name);
	return NULL;
    }
    deblog_info(_("Master host name: %s\n"), hinfo->h_name);
    deblog_info(_("Aliases:\n"));
    for (aliases = hinfo->h_aliases; *aliases; ++aliases)
    {
	deblog_info("\t%s\n", *aliases);
    }
    deblog_info(_("Addresses:\n"));
    for (addrs = hinfo->h_addr_list; *addrs; ++addrs)
    {
	deblog_info(_("\t%s\n"), inet_ntoa(*(struct in_addr *) *addrs));
    }
    return XSTRDUP(inet_ntoa(*(struct in_addr *) hinfo->h_addr_list[0]));
}

const char *nntp_check_response(rw_buf *sbuf, const char *server_name,
	const char *message, const char *allow, deblog_level_t log_level)
{
    char digit[2] = " ";
    const char *response = rw_read_line(sbuf);
    
    if (response)
	digit[0] = response[0];
    if (!response || !strpbrk(digit, allow))
    {
	deblog_printf(log_level, "%s: %s: %s\n", server_name, message,
		response ? response : _("no response from server"));
	return NULL;
    }
    else
    {
	deblog_debug("%s\n", response);
    }
    return response;
}

rw_buf *nntp_connect(const char *server_name, const char *dotquad,
	int port, int thread_idx, BOOL ssl)
{
    int sock;
    rw_buf *buf;
#ifdef SUPPORT_SSL
    static BOOL ssl_initialised = FALSE;
    static SSL_CTX *ssl_ctx = NULL;
    SSL *ssl_dat = NULL;

    /* This seems safest place to call SSL_library_init, so that each process
     * calls it once instead of relying on correct state to be inherited from
     * parent
     */
    if (ssl)
    {
	if (!ssl_initialised)
	{
	    SSL_library_init();
	    ssl_initialised = TRUE;
	}

	if (!ssl_ctx)
	{
	    ssl_ctx = SSL_CTX_new(SSLv23_client_method());
	    if (!ssl_ctx)
	    {
		deblog_critical(_("Unable to create SSL context\n"));
		return NULL;
	    }
	}
    }
#else
    (void) ssl;
#endif

    deblog_info(_("Connecting to %s:%d\n"), dotquad, ntohs(port));
    sock = socket(AF_INET, SOCK_STREAM, 0);
    if (sock != -1)
    {
	struct sockaddr_in addr;

	addr.sin_family = AF_INET;
	addr.sin_port = port;
	inet_aton(dotquad, &addr.sin_addr);
	if (connect(sock, (struct sockaddr *) &addr, sizeof(addr))
		== -1)
	{
	    int terrno = errno;

	    close(sock);
	    errno = terrno;
	    sock = -1;
	}
    }
    else
    {
	deblog_info(_("Couldn't create socket\n"));
    }
    if (sock == -1)
    {
	deblog_critical(_("Can't connect to %s: %s\n"),
		dotquad, strerror(errno));
	return NULL;
    }
#ifdef SUPPORT_SSL
    if (ssl)
    {
	nscurses_show_status(thread_idx, _("Initialising SSL"));
	deblog_message(_("Making socket secure\n"));
	ssl_dat = SSL_new(ssl_ctx);
	if (!ssl_dat)
	{
	    deblog_critical(_("Unable to create SSL object\n"));
	    close(sock);
	    return NULL;
	}
	if (!SSL_set_fd(ssl_dat, sock))
	{
	    deblog_critical(_("Unable to attach SSL object to socket\n"));
	    SSL_free(ssl_dat);
	    close(sock);
	    return NULL;
	}
	if (SSL_connect(ssl_dat) != 1)
	{
	    deblog_critical(_("SSL handshake failed\n"));
	    SSL_free(ssl_dat);
	    close(sock);
	    return NULL;
	}
    }
    buf = rw_create_buf_ssl(sock, ssl_dat, TRUE, FALSE, thread_idx);
#else
    buf = rw_create_buf(sock, TRUE, FALSE, thread_idx);
#endif
    /*
    if (strcmp(dotquad, "127.0.0.1"))
	buf->scrutinise = TRUE;
    */
    if (!nntp_check_response(buf, server_name,
		_("Bad/no response when connecting"),
		"2", deblog_Level_Critical))
    {
	rw_close_and_free(buf);
	return NULL;
    }
    return buf;
}

BOOL nntp_authenticate(rw_buf *buf, const char *server_name,
	const char *user, const char *passwd)
{
    deblog_info(_("Authenticating user %s\n"), user);
    rw_writef(buf, "AUTHINFO USER %s\r\n", user);
    if (!nntp_check_response(buf, server_name, _("User name rejected"), "23",
		deblog_Level_Critical))
    {
	return TRUE;
    }
    rw_writef(buf, "AUTHINFO PASS %s\r\n", passwd);
    return (nntp_check_response(buf, server_name, _("Password rejected"), "2",
		deblog_Level_Critical) == NULL);
}

BOOL nntp_set_mode(rw_buf *buf, const char *server_name, const char *mode)
{
    deblog_info(_("Sending MODE %s\n"), mode);
    rw_writef(buf, "MODE %s\r\n", mode);
    return (nntp_check_response(buf, server_name,
		_("Bad response to MODE command"), "2", deblog_Level_Critical)
	    == NULL);
}

BOOL nntp_enter_group(rw_buf *buf, const char *server_name,
	const char *groupname, long int *first, long int *last,
	BOOL check_response)
{
    const char *response;

    deblog_info(_("Entering group %s\n"), groupname);
    rw_writef(buf, "GROUP %s\r\n", groupname);
    if (check_response)
    {
	if ((response = nntp_check_response(buf, server_name,
			_("Can't enter group"), "2", deblog_Level_Warning))
		== NULL)
	{
	    return TRUE;
	}
	if (first && last)
	{
	    sscanf(response, "%*d %*d %ld %ld", first, last);
	}
    }
    return FALSE;
}
